@@ -589,6 +589,8 @@ class SpellChecker(GObject.Object):
         )
         if not self._enabled:
             return
+        start = start.copy()
+        end = end.copy()
         if self._iter_worker.inside_word(end):
             self._iter_worker.forward_word_end(end)
         if self._iter_worker.inside_word(start) or self._iter_worker.ends_word(start):
@@ -718,9 +720,11 @@ class SpellChecker(GObject.Object):
                     except AttributeError:
                         label.set_alignment(0.0, 0.5)
                     item.add(label)
-                    item.connect(
-                        "activate", lambda *args: self._replace_word(suggestion)
-                    )
+
+                    def _make_on_activate(word):
+                        return lambda *args: self._replace_word(word)
+
+                    item.connect("activate", _make_on_activate(word))
                 else:
                     escaped = suggestion.replace("'", "\\'")
                     item = Gio.MenuItem.new(
